import { world, ItemStack } from "@minecraft/server";
import SelectionBoxes from "../utils/selection_boxes"; // Import the SelectionBoxes class to use it

// Support orientation along both horizontal axes
const pots = {
  x: new SelectionBoxes({ origin: [-8, 0, -8], size: [16, 16, 0] }, { origin: [-8, 0, -8], size: [0, 16, 16] }, { origin: [-8, 0, 8], size: [16, 16, 0] }, { origin: [8, 0, -8], size: [0, 16, 16] }),
  z: new SelectionBoxes({ origin: [8, 0, 8], size: [0, 16, 16] }, { origin: [8, 0, 8], size: [16, 16, 0] }, { origin: [8, 0, -8], size: [0, 16, 16] }, { origin: [-8, 0, 8], size: [16, 16, 0] }),

};

// The state value and sound associated with each plant
const plants = {
  "fb:wall_block_2": {
    value: "wall_tiles_red",
    sound: "block.lantern.break",
  },
  "fb:wall_block_1": {
    value: "wall_tiles_black",
    sound: "block.lantern.break",
  },
  "fb:wall_middle_purple": {
    value: "wall_tiles_purple",
    sound: "block.lantern.break",
  },
  "fb:wall_block_3": {
    value: "wall",
    sound: "block.lantern.break",
  },
  "fb:wall_top_block_2": {
    value: "wall_tiles_red_top",
    sound: "block.lantern.break",
  },
  "fb:wall_down_block_3": {
    value: "wall_tiles",
    sound: "block.lantern.break",
  },
  "fb:wall_down_block_1": {
    value: "wall_down_block_black",
    sound: "block.lantern.break",
  },
  "fb:wall_top_block_1": {
    value: "wall_top_side",
    sound: "block.lantern.break",
  },
  "fb:stage_wall_down": {
    value: "stage_wall_down",
    sound: "block.lantern.break",
  },
  "fb:stage_wall_up": {
    value: "stage_wall_up",
    sound: "block.lantern.break",
  },
  "fb:stage_wall_2": {
    value: "stage_wall_2",
    sound: "block.lantern.break",
  },
  "fb:stage_wall_1": {
    value: "stage_wall_1",
    sound: "block.lantern.break",
  },
  "fb:wall_down_block_2": {
    value: "wall_down_block_2",
    sound: "block.lantern.break",
  },

  ////// wall_down_block_2 ()
  ////// stage_wall_1 ()
  ////// stage_wall_2 ()
  ////// stage_wall_up ()
  ////// stage_wall_down ()
};

// Get the selected pot for the appropriate axis
const getSelectedPot = (e) => pots[e.block.permutation.getState("wiki:axis")].getSelected(e.faceLocation);

const isPotOccupied = (block, pot) => block.permutation.getState(`wiki:pot_${pot}_plant`) !== "none";

const setPotPlant = (block, pot, plant) => block.setPermutation(block.permutation.withState(`wiki:pot_${pot}_plant`, plant));

// Our flower pots even have sound effects!
const playPlantSound = (dimension, location, sound) => dimension.runCommand(`playsound ${sound} @a ${location.x} ${location.z} 0.5`);

// If a pot is not selected (the inbetween area is targeted) or is already filled, the item use is cancelled.
world.beforeEvents.itemUseOn.subscribe((e) => {
  if (e.block.typeId !== "fb:advanced_wall" || !plants[e.itemStack.typeId]) return;

  const selectedPot = getSelectedPot(e);

  if (selectedPot === undefined || isPotOccupied(e.block, selectedPot)) e.cancel = false;
});

// -------------------------------
//    Plant in the selected pot 
// -------------------------------
world.afterEvents.itemUseOn.subscribe((e) => {
  if (e.block.typeId !== "fb:advanced_wall" || !plants[e.itemStack.typeId] || e.source.isSneaking) return;

  const selectedPot = getSelectedPot(e);
  const plant = plants[e.itemStack.typeId];

  setPotPlant(e.block, selectedPot, plant.value);
  playPlantSound(e.block.dimension, e.block.location, plant.sound);
});

// -------------------------------
//  Release plants on destruction
// -------------------------------
function releasePlants(e) {
  const states = (e.brokenBlockPermutation ?? e.explodedBlockPermutation).getAllStates();

  // Array of plant state values e.g. ["cactus", "dandelion"]
  const storedPlants = Object.entries(states)
    .filter(([state, value]) => state.startsWith("wiki:pot") && value !== "none")
    .map(([state, value]) => value);

  if (storedPlants.length === 0) return;

}

world.afterEvents.playerBreakBlock.subscribe((e) => {
  if (e.brokenBlockPermutation.type.id === "wiki:double_flower_pot") releasePlants(e);
});
world.afterEvents.blockExplode.subscribe((e) => {
  if (e.explodedBlockPermutation.type.id === "wiki:double_flower_pot") releasePlants(e);
});
